/*
 * file BasicDetailedFeedback.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2007, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */

package javax.wvcm;

import java.text.MessageFormat;

import javax.wvcm.PropertyRequestItem.PropertyRequest;

public class DetailedFeedback implements Feedback
{
    public static class NestedDetailedFeedback implements Feedback
    {
        private DetailedFeedback _detailedFeedback = null;
        private PropertyRequest _forResult = null;
        private int _begin = 0; // The beginning of the nested interval
        private int _end = 0; // The end of the nested interval
        
        private NestedDetailedFeedback(
                DetailedFeedback detailedFeedback,
                PropertyRequest forResult,
                int end) {
            _detailedFeedback = detailedFeedback;
            _forResult = forResult;
            _begin = detailedFeedback._last;
            _end = end;
          }
          
        public PropertyRequest getPropertyRequestForResult() {
            return _forResult;
        }

        public PropertyRequest getPropertyRequestForModified() {
            return _detailedFeedback._forModified;
        }

        public boolean isAbortRequested() {
            return _detailedFeedback.isAbortRequested();
        }

        public Feedback nest() {
            return nest(null, 0);
        }
        
        public Feedback nest(PropertyRequest forResult) {
            return nest(forResult, 0);
        }

        public Feedback nest(int percentCompleted) {
            return nest(null, percentCompleted);
        }

        private boolean equalPropertyRequest(PropertyRequest pr1, PropertyRequest pr2) {
            if (pr1 == pr2) {
                return true;
            }
            if (pr1 != null) {
                return pr1.equals(pr2);
            }
            return false;
        }
            
        public Feedback nest(PropertyRequest forResult, int percentToBeCompleted) {
            if (equalPropertyRequest(forResult, _forResult) && (_end - _begin) == 0) {
                return this;
            }
            int end = _begin + (percentToBeCompleted * (_end - _begin) / 100);
            return new NestedDetailedFeedback(_detailedFeedback, forResult, end);
        }
        
        public String format(String fmt, Object...arguments) {
            return MessageFormat.format(fmt, arguments);
        }

        public void notifyActive(String message) {
            _detailedFeedback.notifyActive(message);
        }

        public void notifyWarning(String message) {
            _detailedFeedback.notifyWarning(message);
        }

        public void notifyIsModified(Resource resource) {
            _detailedFeedback.notifyIsModified(resource);
        }

        public void notifyPercentComplete(int percentComplete) {
            int done = _begin + (percentComplete * (_end - _begin) / 100);
            if (done > _detailedFeedback._last) {
                _detailedFeedback.notifyPercentComplete(done);
            }
        }
        
        public void abortRequested(boolean isAbortRequested) {
            _detailedFeedback._isAbortRequested = isAbortRequested;
        }

        public void setPropertyRequestForResult(PropertyRequest propertyRequestForResult) {
            _forResult = propertyRequestForResult;
        }

        public void setPropertyRequestForModified(PropertyRequest propertyRequestForModified) {
            _detailedFeedback._forModified = propertyRequestForModified;
        }

    }
    private static final Feedback nullDetailedFeedback = new DetailedFeedback();
    
    private PropertyRequest _forResult = null;
    private PropertyRequest _forModified = null;
    private boolean _isAbortRequested = false;
    protected int _last = 0; // The last percentage reported
    
    public DetailedFeedback() {
    }
      
    /**
     * Create a non-null Feedback object.
     * @return a non-null Feedback object.
     */
    public static Feedback fb(Feedback feedback) {
        if (feedback == null) {
            return nullDetailedFeedback;
        }
        if (feedback instanceof Feedback) {
            return (Feedback)feedback;
        }
        throw new RuntimeException("Unexpected feedback type"); //$NON-NLS-1$
    }

    public PropertyRequest getPropertyRequestForResult() {
        return _forResult;
    }

    public PropertyRequest getPropertyRequestForModified() {
        return _forModified;
    }

    public boolean isAbortRequested() {
        return _isAbortRequested;
    }

    public Feedback nest() {
        return nest(null, 0);
    }
    
    public Feedback nest(PropertyRequest forResult) {
        return nest(forResult, 0);
    }

    public Feedback nest(int percentCompleted) {
        return nest(null, percentCompleted);
    }

    public Feedback nest(PropertyRequest forResult, int percentToBeCompleted) {
        int end = _last;
        if (end < percentToBeCompleted) {
            end = percentToBeCompleted;
        }
        return new NestedDetailedFeedback(this, forResult, end);
    }

    public String format(String fmt, Object...arguments) {
        return MessageFormat.format(fmt, arguments);
    }

    public void notifyActive(String message) {
    }

    public void notifyWarning(String message) {
    }

    public void notifyIsModified(Resource resource) {
    }

    public void notifyPercentComplete(int percentComplete) {
        _last = percentComplete;
    }
    
    public void abortRequested(boolean isAbortRequested) {
        _isAbortRequested = isAbortRequested;
    }

    public void setPropertyRequestForResult(PropertyRequest propertyRequestForResult) {
        _forResult = propertyRequestForResult;
    }

    public void setPropertyRequestForModified(PropertyRequest propertyRequestForModified) {
        _forModified = propertyRequestForModified;
    }

}

